#include <GL/glut.h>
#include <math.h>
#include <iostream>
#include "4d.h"

#define center -0.86

using namespace std;
using namespace math4d;

int WW = 1024;
int WH = 768;
int mx, my;
int MouseStatus;
double angle = 0.01;
double pi;
int sw = 0;

vector x1(1,0,0,0);
vector x2(0,1,0,0);
vector x3(0,0,1,0);
vector x4(0,0,0,1);

OPER PROJ;
OPER TURN;

void TTC (double& x, double& y, double a, double b) //Transform To Continious
{
	double pixlen;
	
	if (WW > WH)
		pixlen = 2.0/WH;
	else
		pixlen = 2.0/WW;	
	x = (a - WW/2.0)*pixlen;
	y = (b - WH/2.0)*pixlen;
}
void TTD (double x, double y, double& a, double& b) //Transform To Discret
{
	double pixlen;
	
	if (WW > WH)
		pixlen = 2.0/WH;
	else
		pixlen = 2.0/WW;
	a = x/pixlen + WW/2.0;
	b = y/pixlen + WH/2.0;
}

void IDrawPart(float r, float g, float b, double M, double RadM, int sw = 0, int inter = 10)
{
	static double x, y, AngleStep, Angle;
	static int i;
	
	AngleStep = pi/2.0/inter;
	Angle = pi/4*M - AngleStep;
	
	sw ? glColor3f(1, 1, 1) : glColor3f(r, g, b);
	glBegin(GL_TRIANGLE_FAN);
	TTD(center, center, x, y);
	glVertex2d(x, y);
	for (i = 0; i <= inter; ++i)
	{
		Angle+=AngleStep;
		TTD(cos(Angle)*RadM+center, sin(Angle)*RadM+center, x, y);
		glVertex2d(x, y);
	}
	glEnd();
}

void IDrawLine(float r, float g, float b, double M, double RadM)
{
	static double x, y;
		
	glBegin(GL_LINES);
		glColor3f(r, g, b);
		TTD(center, center, x, y);
		glVertex2d(x, y);
		TTD(cos(pi/4*M)*RadM+center, sin(pi/4*M)*RadM+center, x, y);
		glVertex2d(x, y);
	glEnd();
	glPointSize(5);
	glBegin(GL_POINTS);
		glVertex2d(x, y);
	glEnd();
}

void DrawVector(vector& x, const int r, const int g, const int b)
{
	static double X, Y;
	glColor3f(r, g, b);
	glBegin(GL_LINES);
		glVertex2d(WW/2.0, WH/2.0);
		TTD (x.v[0], x.v[1], X, Y);
		glVertex2d(X, Y);
	glEnd();
	glBegin(GL_POINTS);
		glVertex2d(X, Y);
	glEnd();
}

void PassiveMotion (int x, int y)
{
	static double X,Y, temp;
	static int swOld;
	swOld = sw;
	y = WH-y;
	TTC (X, Y, x, y);
	temp = (pow(X-center, 2)+(pow(Y-center, 2)));
	if (temp < 0.01)
		if (temp < 0.004225)
			if (((X > center) && (Y > center)) || ((X <= center) && (Y <= center)))
				sw = 5;
			else
				sw = 6;
		else
		{
			temp = Y-X;
			if (X+Y > 2*center)
				if (temp > 0)
					sw = 1;
				else
					sw = 4;
			else
				if (temp > 0)
					sw = 2;
				else
					sw = 3;
		}
	else
		sw = 0;

	if (sw != swOld)
	{
		glutPostRedisplay();
	}
}

void PressedMotion (int x, int y)
{
	TURN.LoadTurn(angle*(mx - x), 0, 0, 1);
	mx = x;
	
	TURN(x1);
	TURN(x2);
	TURN(x3);
	TURN(x4);
	glutPostRedisplay();
}

void Mouse (int button, int state, int x, int y)
{
	if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_DOWN))
	{
		mx = x;
		glutMotionFunc(PressedMotion);
		switch (sw)
		{
			case 0:
				glutMotionFunc(NULL);
				break;
			case 1:
				TURN.LoadTurn(1.0, 1, 2);
				break;
			case 2:
				TURN.LoadTurn(1.0, 2, 3);
				break;
			case 3:
				TURN.LoadTurn(1.0, 3, 4);
				break;
			case 4:
				TURN.LoadTurn(1.0, 1, 4);
				break;
			case 5:
				TURN.LoadTurn(1.0, 1, 3);
				break;
			case 6:
				TURN.LoadTurn(1.0, 2, 4);
				break;	
		}
	}
	if ((button == GLUT_RIGHT_BUTTON) && (state == GLUT_DOWN))
	{
		x1.print();
		x2.print();
		x3.print();
		x4.print();
		cout << '\n';
	}
}

void InterfaceDraw(const int sw)
{
	static int A[] = {0,0,0,0,0,0,0};
	
	A[sw] = 1;
	
	IDrawPart(1, 0, 0, 1, 0.1, A[1]);
	IDrawPart(0, 1, 0, 3, 0.1, A[2]);
	IDrawPart(0, 0, 1, 5, 0.1, A[3]);
	IDrawPart(1, 1, 0, 7, 0.1, A[4]);
	
	IDrawLine(1, 1, 0, 3, 0.141);
	IDrawLine(0, 1, 1, 5, 0.141);
	IDrawLine(0.5, 1, 0.5, 7, 0.141);
	IDrawLine(1, 0.5, 0, 1, 0.141);
	
	IDrawPart(0, 0, 0, 0, 0.07);
	IDrawPart(0, 0, 0, 2, 0.07);
	IDrawPart(0, 0, 0, 4, 0.07);
	IDrawPart(0, 0, 0, 6, 0.07);
	
	IDrawPart(1, 0.5, 0, 0, 0.065, A[5]);
	IDrawPart(0.5, 1, 0.8, 2, 0.065, A[6]);
	IDrawPart(1, 0.5, 0, 4, 0.065, A[5]);
	IDrawPart(0.5, 1, 0.8, 6, 0.065, A[6]);
	A[sw] = 0;
}

void Display()
{
	glClear(GL_COLOR_BUFFER_BIT);

	InterfaceDraw(sw);
	
	DrawVector(x1, 1, 1, 0);
	DrawVector(x2, 0, 1, 1);
	DrawVector(x3, 1, 0, 1);
	DrawVector(x4, 1, 1, 1);

	glutSwapBuffers();
}

void Reshape(int width, int height)
{
	WW=width;
	WH=height;

	glViewport(0, 0, WW, WH);

	glLoadIdentity();
	gluOrtho2D(0, WW-1, 0, WH-1);
	glutPostRedisplay();
}

int main(int argc, char* argv[])
{
//Инициализация математики
	pi = acos(-1);

	PROJ.LoadProj(1,2);
	TURN.LoadTurn(angle, 1,2);
//Тестовая секция
	
	
//Инициализация графики
	glutInit (&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);
	glutInitWindowSize(WW, WH);
	glutInitWindowPosition(0, 0);
	glutCreateWindow("4d -> 2d projection viewer");//Создать окно! Пока невидимое Конец начальной инициализациии...

//Еще что-то инициализируется
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	//glEnable(GL_POLYGON_SMOOTH);
	glEnable(GL_POINT_SMOOTH);
	glEnable(GL_LINE_SMOOTH);
	
	glClearColor(0.0, 0.0, 0.0, 1.0);
	
//Установка функций обратного вызова.
	glutDisplayFunc(Display);
	//glutKeyboardFunc(Keyboard);
	//glutSpecialFunc(KeyboardSpecial);
	glutReshapeFunc(Reshape);
	glutMouseFunc(Mouse);
	//glutMotionFunc(MouseMotion);
	glutPassiveMotionFunc(PassiveMotion);
//Поехали!
	glutMainLoop();	
	return 0;
}
